local _G = getfenv(0)

local YurrCombatLog = LibStub("AceAddon-3.0"):NewAddon("YurrCombatLog", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("YurrCombatLog", true)

local debugf = tekDebug and tekDebug:GetFrame("YurrCombatLog")
local function Debug(...) if debugf then debugf:AddMessage(string.join(", ", ...)) end end

local CombatLogQuickButtonFrameProgressBar
--local CombatLogUpdateFrame = CreateFrame("Frame", "CombatLogUpdateFrame", UIParent)

local prefixList
local suffixList
local spellList
local schools = {
	[SCHOOL_MASK_PHYSICAL]						= STRING_SCHOOL_PHYSICAL,
	[SCHOOL_MASK_HOLY]							= STRING_SCHOOL_HOLY,
	[SCHOOL_MASK_FIRE]							= STRING_SCHOOL_FIRE,
	[SCHOOL_MASK_NATURE]						= STRING_SCHOOL_NATURE,
	[SCHOOL_MASK_FROST]							= STRING_SCHOOL_FROST,
	[SCHOOL_MASK_SHADOW]						= STRING_SCHOOL_SHADOW,
	[SCHOOL_MASK_ARCANE]						= STRING_SCHOOL_ARCANE,
	-- Physical and a Magical
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE]	= STRING_SCHOOL_FLAMESTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FROST]	= STRING_SCHOOL_FROSTSTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_ARCANE]	= STRING_SCHOOL_SPELLSTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_STORMSTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWSTRIKE,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_HOLY]	= STRING_SCHOOL_HOLYSTRIKE,
	-- Two Magical Schools
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST]		= STRING_SCHOOL_FROSTFIRE,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_ARCANE]		= STRING_SCHOOL_SPELLFIRE,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_NATURE]		= STRING_SCHOOL_FIRESTORM,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_SHADOW]		= STRING_SCHOOL_SHADOWFLAME,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYFIRE,
	[SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE]	= STRING_SCHOOL_SPELLFROST,
	[SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_FROSTSTORM,
	[SCHOOL_MASK_FROST + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWFROST,
	[SCHOOL_MASK_FROST + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYFROST,
	[SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_SPELLSTORM,
	[SCHOOL_MASK_ARCANE + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SPELLSHADOW,
	[SCHOOL_MASK_ARCANE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_DIVINE,
	[SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWSTORM,
	[SCHOOL_MASK_NATURE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYSTORM,
	[SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_SHADOWLIGHT,
	-- Three or more schools
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]																						= STRING_SCHOOL_ELEMENTAL,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]											= STRING_SCHOOL_CHROMATIC,
	[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]						= STRING_SCHOOL_MAGIC,
	[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]	= STRING_SCHOOL_CHAOS
}
local powerTypes = {
	[-2] = string.lower(HEALTH),
	[0] = string.lower(MANA),
	[1] = string.lower(RAGE),
	[2] = string.lower(FOCUS),
	[3] = string.lower(ENERGY),
	[4] = string.lower(format("%s%s", PET, HAPPINESS)),
	[5] = string.lower(RUNES),
	[6] = string.lower(RUNIC_POWER),
	[7] = string.lower(SOUL_SHARDS),
	[8] = string.lower(ECLIPSE),
	[9] = string.lower(HOLY_POWER),
}
local classColors = {}

local defaults = {
	profile = {
		version = 0,
		outputFrame = "ChatFrame2",
		unitColors = {
			[1297] = "8080FF",		-- self
			[4991] = "804D26",		-- pet
			[1394] = "A6FFA6",		-- party
			[1396] = "FF7038",		-- raid
			[9087] = "404080",		-- guardian
			[131072] = "CC54D9",	-- focus
			[20334] = "FF75FF",		-- enemy players/npcs
			[3864] = "FFFFFF",		-- outsider
		},
		spellColors = {
			[SCHOOL_MASK_PHYSICAL]						= "FFFFFF",
			[SCHOOL_MASK_HOLY]							= "FFFF4D",
			[SCHOOL_MASK_FIRE]							= "FF262E",
			[SCHOOL_MASK_NATURE]						= "66FF66",
			[SCHOOL_MASK_FROST]							= "4D4DE6",
			[SCHOOL_MASK_SHADOW]						= "FFB3FF",
			[SCHOOL_MASK_ARCANE]						= "BFBFBF",
			-- Physical and a Magical
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE]	= "FF9397",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FROST]	= "A6A6F3",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_ARCANE]	= "DFDFDF",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_NATURE]	= "B3FFB3",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_SHADOW]	= "FFD9FF",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_HOLY]	= "FFFFD3",
			-- Two Magical Schools
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST]		= "A63A8A",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_ARCANE]		= "DF7377",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_NATURE]		= "B3934A",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_SHADOW]		= "FF6D97",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_HOLY]		= "FF933E",
			[SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE]	= "8686D3",
			[SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]	= "5AA6A6",
			[SCHOOL_MASK_FROST + SCHOOL_MASK_SHADOW]	= "A680F3",
			[SCHOOL_MASK_FROST + SCHOOL_MASK_HOLY]		= "A6A69A",
			[SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE]	= "93DF93",
			[SCHOOL_MASK_ARCANE + SCHOOL_MASK_SHADOW]	= "DFB9DF",
			[SCHOOL_MASK_ARCANE + SCHOOL_MASK_HOLY]		= "DFDF86",
			[SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]	= "B3D9B3",
			[SCHOOL_MASK_NATURE + SCHOOL_MASK_HOLY]		= "B3FF5A",
			[SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]		= "FFD9A6",
			-- Three or more schools
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]																						= "917B7E",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]											= "B094A5",
			[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]						= "BDA696",
			[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]	= "C7B3A5"
		},
		otherColors = {
			heal = "FFFF4D",
			miss = "FF7F7F",
			buff = "33FF33",
			debuff = "FF3333",
			skill = "FFFF66",
			timestampColor = "979797",
			
		},
		blacklist = {},
		enableWhitelist = false,
		whitelist = {},
		watches = {},
		braces = {
			preTimestamp = "[",
			postTimestamp = "]",
			preSource = "[",
			postSource = "]",
			preSpell = "[",
			postSpell = "]",
			preDest = "[",
			postDest = "]",
			preAmount = "[",
			postAmount = "]",
			preAmountDots = "~",
			postAmountDots = "~",
			preSchoolText = "[",
			postSchoolText = "]",
			preCrit = "*",
			postCrit = "*",
		},
		braceColors = {
			Timestamp = "979797",
			Source = "FF75FF",
			Spell = "FF75FF",
			Dest = "FF75FF",
			Amount = "FF75FF",
			AmountDots = "FF75FF",
			SchoolText = "FF75FF",
		},
		
		contextMenu = true,
		quickButtons = true,
		
		timestamp = true,
		colorTimestamp = true,
		braceTimestamp = true,
		colorTimestampBrace = true,
		timestampFormat = "%X",
		
		colorUnitByClass = false,
		
		braceSrcUnit = true,
		colorSrc = true,
		colorSrcBraces = false,
		
		prefix = true,
		colorPrefix = true,
		["++"] = "++",
		["--"] = "--",
		["~~"] = "~~",
		
		braceSpell = false,
		colorSpell = true,
		colorSpellBraces = false,
		
		suffixString = true,
		resultInSuffix = true,
		colorSuffix = true,
		
		braceDestUnit = true,
		colorDest = true,
		colorDestBraces = false,
		
		braceAmount = false,
		colorAmount = true,
		colorAmountBraces = false,
		braceAmountDots = true,
		colorAmountDotBraces = false,
		markCrits = true,
		
		overhealing = true,
		overkill = true,
		
		resultString = true,
		
		powerType = true,
		
		schoolText = false,
		colorSchoolText = true,
		braceSchoolText = false,
		colorSchoolTextBraces = false,
		
		format = {
			ABSORB = "(A%d)",
			BLOCK = BLOCK,
			BLOCK_VALUE = "(B%d)",
			CRITICAL = L["Crit"],
			CRUSHING = L["Crush"],
			DEFLECT = DEFLECT,
			DODGE = DODGE,
			EVADE = L["Evade"],
			GLANCING = L["Glancing"],
			IMMUNE = SPELL_FAILED_IMMUNE,
			MISS = MISS,
			PARRY = PARRY,
			REFLECT = REFLECT,
			RESIST = "(R%d)",
			VULNERABILITY = "(V%d)",
		},
		filters = {
			[1] = {
				name = "By me",
				active = true,
				ENVIRONMENTAL_DAMAGE = true,
				SWING_DAMAGE = true,
				SWING_MISSED = true,
				RANGE_DAMAGE = true,
				RANGE_MISSED = true,
				SPELL_CAST_START = false,
				SPELL_CAST_SUCCESS = false,
				SPELL_CAST_FAILED = false,
				SPELL_MISSED = true,
				SPELL_DAMAGE = true,
				SPELL_HEAL = true,
				SPELL_ENERGIZE = true,
				SPELL_DRAIN = true,
				SPELL_LEECH = true,
				SPELL_SUMMON = true,
				SPELL_RESURRECT = true,
				SPELL_CREATE = true,
				SPELL_INSTAKILL = true,
				SPELL_INTERRUPT = true,
				SPELL_EXTRA_ATTACKS = true,
				SPELL_DURABILITY_DAMAGE = false,
				SPELL_DURABILITY_DAMAGE_ALL = false,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_APPLIED_DOSE = true,
				SPELL_AURA_REMOVED = true,
				SPELL_AURA_REMOVED_DOSE = true,
				SPELL_AURA_BROKEN = true,
				SPELL_AURA_BROKEN_SPELL = true,
				SPELL_AURA_REFRESH = true,
				SPELL_DISPEL = true,
				SPELL_STOLEN = true,
				ENCHANT_APPLIED = true,
				ENCHANT_REMOVED = true,
				SPELL_PERIODIC_MISSED = true,
				SPELL_PERIODIC_DAMAGE = true,
				SPELL_PERIODIC_HEAL = true,
				SPELL_PERIODIC_ENERGIZE = true,
				SPELL_PERIODIC_DRAIN = true,
				SPELL_PERIODIC_LEECH = true,
				SPELL_DISPEL_FAILED = true,
				DAMAGE_SHIELD = true,
				DAMAGE_SHIELD_MISSED = true,
				DAMAGE_SPLIT = true,
				PARTY_KILL = true,
				UNIT_DIED = true,
				UNIT_DESTROYED = true,
				SPELL_BUILDING_DAMAGE = true,
				SPELL_BUILDING_HEAL = true,
				UNIT_DISSIPATES = true,
				sourceUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
				destUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = false,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = false,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = false,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = false,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
			},
			[2] = {
				name = "To me",
				active = true,
				ENVIRONMENTAL_DAMAGE = true,
				SWING_DAMAGE = true,
				SWING_MISSED = true,
				RANGE_DAMAGE = true,
				RANGE_MISSED = true,
				SPELL_CAST_START = false,
				SPELL_CAST_SUCCESS = false,
				SPELL_CAST_FAILED = false,
				SPELL_MISSED = true,
				SPELL_DAMAGE = true,
				SPELL_HEAL = true,
				SPELL_ENERGIZE = true,
				SPELL_DRAIN = true,
				SPELL_LEECH = true,
				SPELL_SUMMON = true,
				SPELL_RESURRECT = true,
				SPELL_CREATE = true,
				SPELL_INSTAKILL = true,
				SPELL_INTERRUPT = true,
				SPELL_EXTRA_ATTACKS = true,
				SPELL_DURABILITY_DAMAGE = false,
				SPELL_DURABILITY_DAMAGE_ALL = false,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_APPLIED_DOSE = true,
				SPELL_AURA_REMOVED = true,
				SPELL_AURA_REMOVED_DOSE = true,
				SPELL_AURA_BROKEN = true,
				SPELL_AURA_BROKEN_SPELL = true,
				SPELL_AURA_REFRESH = true,
				SPELL_DISPEL = true,
				SPELL_STOLEN = true,
				ENCHANT_APPLIED = true,
				ENCHANT_REMOVED = true,
				SPELL_PERIODIC_MISSED = true,
				SPELL_PERIODIC_DAMAGE = true,
				SPELL_PERIODIC_HEAL = true,
				SPELL_PERIODIC_ENERGIZE = true,
				SPELL_PERIODIC_DRAIN = true,
				SPELL_PERIODIC_LEECH = true,
				SPELL_DISPEL_FAILED = true,
				DAMAGE_SHIELD = true,
				DAMAGE_SHIELD_MISSED = true,
				DAMAGE_SPLIT = true,
				PARTY_KILL = true,
				UNIT_DIED = true,
				UNIT_DESTROYED = true,
				SPELL_BUILDING_DAMAGE = true,
				SPELL_BUILDING_HEAL = true,
				UNIT_DISSIPATES = true,
				sourceUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = false,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = false,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = false,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = false,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
				destUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
			},
			[3] = {
				name = "Everything",
				active = false,
				ENVIRONMENTAL_DAMAGE = true,
				SWING_DAMAGE = true,
				SWING_MISSED = true,
				RANGE_DAMAGE = true,
				RANGE_MISSED = true,
				SPELL_CAST_START = true,
				SPELL_CAST_SUCCESS = true,
				SPELL_CAST_FAILED = true,
				SPELL_MISSED = true,
				SPELL_DAMAGE = true,
				SPELL_HEAL = true,
				SPELL_ENERGIZE = true,
				SPELL_DRAIN = true,
				SPELL_LEECH = true,
				SPELL_SUMMON = true,
				SPELL_RESURRECT = true,
				SPELL_CREATE = true,
				SPELL_INSTAKILL = true,
				SPELL_INTERRUPT = true,
				SPELL_EXTRA_ATTACKS = true,
				SPELL_DURABILITY_DAMAGE = true,
				SPELL_DURABILITY_DAMAGE_ALL = true,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_APPLIED_DOSE = true,
				SPELL_AURA_REMOVED = true,
				SPELL_AURA_REMOVED_DOSE = true,
				SPELL_AURA_BROKEN = true,
				SPELL_AURA_BROKEN_SPELL = true,
				SPELL_AURA_REFRESH = true,
				SPELL_DISPEL = true,
				SPELL_STOLEN = true,
				ENCHANT_APPLIED = true,
				ENCHANT_REMOVED = true,
				SPELL_PERIODIC_MISSED = true,
				SPELL_PERIODIC_DAMAGE = true,
				SPELL_PERIODIC_HEAL = true,
				SPELL_PERIODIC_ENERGIZE = true,
				SPELL_PERIODIC_DRAIN = true,
				SPELL_PERIODIC_LEECH = true,
				SPELL_DISPEL_FAILED = true,
				DAMAGE_SHIELD = true,
				DAMAGE_SHIELD_MISSED = true,
				DAMAGE_SPLIT = true,
				PARTY_KILL = true,
				UNIT_DIED = true,
				UNIT_DESTROYED = true,
				SPELL_BUILDING_DAMAGE = true,
				SPELL_BUILDING_HEAL = true,
				UNIT_DISSIPATES = true,
				sourceUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = true,
					COMBATLOG_OBJECT_AFFILIATION_RAID = true,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = true,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = true,
					COMBATLOG_OBJECT_REACTION_HOSTILE = true,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = true,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = true,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = true,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = true,
					COMBATLOG_OBJECT_FOCUS = true,
					COMBATLOG_OBJECT_MAINTANK = true,
					COMBATLOG_OBJECT_MAINASSIST = true,
					COMBATLOG_OBJECT_RAIDTARGET1 = true,
					COMBATLOG_OBJECT_RAIDTARGET2 = true,
					COMBATLOG_OBJECT_RAIDTARGET3 = true,
					COMBATLOG_OBJECT_RAIDTARGET4 = true,
					COMBATLOG_OBJECT_RAIDTARGET5 = true,
					COMBATLOG_OBJECT_RAIDTARGET6 = true,
					COMBATLOG_OBJECT_RAIDTARGET7 = true,
					COMBATLOG_OBJECT_RAIDTARGET8 = true,
					COMBATLOG_OBJECT_NONE = true,
				},
				destUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = true,
					COMBATLOG_OBJECT_AFFILIATION_RAID = true,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = true,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = true,
					COMBATLOG_OBJECT_REACTION_HOSTILE = true,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = true,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = true,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = true,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = true,
					COMBATLOG_OBJECT_FOCUS = true,
					COMBATLOG_OBJECT_MAINTANK = true,
					COMBATLOG_OBJECT_MAINASSIST = true,
					COMBATLOG_OBJECT_RAIDTARGET1 = true,
					COMBATLOG_OBJECT_RAIDTARGET2 = true,
					COMBATLOG_OBJECT_RAIDTARGET3 = true,
					COMBATLOG_OBJECT_RAIDTARGET4 = true,
					COMBATLOG_OBJECT_RAIDTARGET5 = true,
					COMBATLOG_OBJECT_RAIDTARGET6 = true,
					COMBATLOG_OBJECT_RAIDTARGET7 = true,
					COMBATLOG_OBJECT_RAIDTARGET8 = true,
					COMBATLOG_OBJECT_NONE = true,
				},
			},
		},
	}
}

function YurrCombatLog:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("YurrCombatLogDB", defaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	self:RegisterChatCommand("yurrcombatlog", "showoptionsgui")
	self:RegisterChatCommand("ycl", "showoptionsgui")
	
	if self.db.profile.version < 0.66 then
		self.db:ResetProfile()
	end
	self.db.profile.version = tonumber(GetAddOnMetadata("YurrCombatLog", "Version"))
	
	self:InitQuickButtons()
	
	prefixList = {
		["_AURA_APPLIED"] = self.db.profile["++"],
		["_AURA_APPLIED_DOSE"] = self.db.profile["++"],
		["_AURA_REMOVED"] = self.db.profile["--"],
		["_AURA_REMOVED_DOSE"] = self.db.profile["--"],
		["_AURA_REFRESH"] = self.db.profile["~~"],
		["_CAST_START"] = L.begins,
		["_CAST_SUCCESS"] = L.casts,
		["_CREATE"] = L.create,
		["_INSTAKILL"] = L.instantkill,
		["_AURA_BROKEN"] = L.broke,
		["_AURA_BROKEN_SPELL"] = L.broke
	}
	suffixList = {
		["_HEAL"] = L.Heal,
		["_PERIODIC_HEAL"] = L.Regen,
		["_PERIODIC_DAMAGE"] = L.Dot,
		["_PERIODIC_MISSED"] = L.Dot,
		["_DISPEL"] = L.Dispel,
		["_DISPEL_FAILED"] = L.Dispel,
		["_INTERRUPT"] = L.Interrupt,
		["_SUMMON"] = L.Summon,
		["_RESURRECT"] = L.Resurrect,
		["MISS"] = self.db.profile.format.MISS
	}
	spellList = {
		["_DISPEL"] = self.db.profile.spellColors[school],
		["_AURA_REMOVED"] = self.db.profile.otherColors.skill,
		["_AURA_REMOVED_DOSE"] = self.db.profile.otherColors.skill,
		["_MISSED"] = self.db.profile.otherColors.miss,
		["BUFF"] = self.db.profile.otherColors.buff,
		["DEBUFF"] = self.db.profile.otherColors.debuff,
		["_HEAL"] = self.db.profile.otherColors.heal,
		["_PERIODIC_HEAL"] = self.db.profile.otherColors.heal
	}
	
	local function RGBPercToHex(r, g, b)
		r = r <= 1 and r >= 0 and r or 0
		g = g <= 1 and g >= 0 and g or 0
		b = b <= 1 and b >= 0 and b or 0
		return string.format("%02x%02x%02x", r*255, g*255, b*255)
	end
	for class, color in pairs(RAID_CLASS_COLORS) do
		classColors[class] = RGBPercToHex(color.r, color.g, color.b)
	end
end

function YurrCombatLog:RefreshConfig()
	self:ReInitQuickButtons()
	self.db.profile.version = tonumber(GetAddOnMetadata("YurrCombatLog", "Version"))
end

-- local FilterText
function YurrCombatLog:InitQuickButtons()
	-- CreateFrame("EditBox", "YCLFilterBox", UIParent, "InputBoxTemplate")
	-- YCLFilterBox:SetAutoFocus(false)

	-- local function find(this)
		-- local text = this:GetText()
		
		-- if text == "" then
			-- FilterText = nil
			-- self:setFilter()
		-- else
			-- FilterText = text
			-- CombatLogResetFilter()
			-- self:Blizzard_CombatLog_Refilter()
		-- end
		
		-- this:ClearFocus()
	-- end

	-- YCLFilterBox:SetScript("OnEnterPressed", find)
	-- YCLFilterBox:SetScript("OnEscapePressed", YCLFilterBox.ClearFocus)
	-- YCLFilterBox:SetScript("OnEditFocusLost", YCLFilterBox.ClearHighlight)
	-- YCLFilterBox:SetScript("OnEditFocusGained", YCLFilterBox.HighlightText)
	
	CreateFrame("Button", "YCLFlyOut")
	YCLFlyOut:SetNormalTexture("Interface/PaperDollInfoFrame/UI-GearManager-FlyoutButton")
	YCLFlyOut:SetHighlightTexture("Interface/PaperDollInfoFrame/UI-GearManager-FlyoutButton")
	YCLFlyOut:SetParent(_G[self.db.profile.outputFrame .. "Tab"])
	YCLFlyOut:SetPoint("TOPLEFT", _G[self.db.profile.outputFrame .. "Tab"], "TOPLEFT", -1, 2)
	YCLFlyOut:SetPoint("BOTTOMRIGHT", _G[self.db.profile.outputFrame .. "Tab"], "TOPRIGHT", -1, -13)
	local function setNormal()
		YCLFlyOut:GetNormalTexture():SetTexCoord(0.0, 1, 0.0, 0.5)
		YCLFlyOut:GetHighlightTexture():SetTexCoord(0.0, 1, 0.5, 1.0)
	end
	local function setOpen()
		YCLFlyOut:GetNormalTexture():SetTexCoord(1, .5, 1, -1.2246063538224e-016, 1.1102230246252e-016, .5, 0, -1.1438332919722e-017)
		YCLFlyOut:GetHighlightTexture():SetTexCoord(1, 1, 1, 0.5, 0, 1, 0, .5)
	end
	setNormal()
	YCLFlyOut:SetScript("OnClick",
		function()
			if YCLQuickButtonFrame:IsShown() then
				YCLQuickButtonFrame:Hide()
				setNormal()
			else
				YCLQuickButtonFrame:Show()
				setOpen()
			end
		end
	)
	
	local function updateGTT(QuickButton, k, v)
		GameTooltip:SetOwner(QuickButton, "ANCHOR_TOP")
		local state = self.db.profile.filters[k].active
		local enabled
		if not state then
			enabled = L.on
		else
			enabled = L.off
		end
		GameTooltip:SetText(format(L.clickToToggle, v.name, enabled))
		GameTooltip:Show()
	end
	
	CreateFrame("Frame", "YCLQuickButtonFrame")
	YCLQuickButtonFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background"})
	YCLQuickButtonFrame:SetBackdropColor(0, 0, 0)
	YCLQuickButtonFrame:SetParent(YCLFlyOut)
	YCLQuickButtonFrame:SetPoint("TOPLEFT", _G[self.db.profile.outputFrame .. "Tab"], "TOPLEFT", 0, 18)
	YCLQuickButtonFrame:SetPoint("BOTTOMRIGHT", _G[self.db.profile.outputFrame], "TOPRIGHT", 0, 36)
	YCLQuickButtonFrame:Hide()
	if not self.db.profile.quickButtons then YCLFlyOut:Hide() end

	-- YCLFilterBox:SetPoint("TOPLEFT", YCLQuickButtonFrame, "TOPLEFT", 5, 24)
	-- YCLFilterBox:SetPoint("BOTTOMRIGHT", YCLQuickButtonFrame, "TOPRIGHT", 0, 0)
	
	local width = 0
	local limit = YCLQuickButtonFrame:GetWidth() - 15
	local dropdownStart
	for k,v in ipairs(self.db.profile.filters) do
		if v then
			if width >= limit then
				dropdownStart = k
				return
			end
			local QuickButton = CreateFrame("Button")
			
			local QuickButtonText = QuickButton:CreateFontString("QuickButton")
			QuickButtonText:SetFont("Fonts\\FRIZQT__.TTF", 11)
			QuickButtonText:SetText(v.name)
			QuickButtonText:SetPoint("CENTER", QuickButton)
			if v.active then
				QuickButtonText:SetVertexColor(1, 1, 1 , 1)
			else
				QuickButtonText:SetVertexColor(1, 1, 1 , 0.5)
			end
			
			QuickButton:SetPoint("TOPLEFT", YCLQuickButtonFrame, "TOPLEFT", width + 8, 0)
			
			width = width + QuickButtonText:GetWidth() + 8
			
			QuickButton:SetPoint("BOTTOMRIGHT", YCLQuickButtonFrame, "BOTTOMLEFT", width + 8, 2)
			QuickButton:SetScript("OnClick", function()
				for k2,v2 in ipairs(self.db.profile.filters) do
					if v2 then
						if v.name == v2.name then
							v2.active = not v2.active
							if v2.active then
								QuickButtonText:SetVertexColor(1, 1, 1 , 1)
							else
								QuickButtonText:SetVertexColor(1, 1, 1 , 0.5)
							end
						end
					end
				end
				updateGTT(QuickButton, k, v)
				self:setFilter()
			end)
			QuickButton:SetScript("OnEnter", function()
				updateGTT(QuickButton, k, v)
			end)
			QuickButton:SetScript("OnLeave", function() GameTooltip:Hide() end)
			QuickButton:SetParent(YCLQuickButtonFrame)
			QuickButtonText:SetParent(QuickButton)
			
			if k == #self.db.profile.filters then return end
		end
	end
end
function YurrCombatLog:ReInitQuickButtons()
	if YCLQuickButtonFrame then
		YCLQuickButtonFrame:Hide()
		YCLQuickButtonFrame = nil
	end
	self:InitQuickButtons()
end

local s = false
function YurrCombatLog:showoptionsgui()
	if not IsAddOnLoaded("YurrCombatLogOptions") then
		local _, reason = LoadAddOn("YurrCombatLogOptions")
		local _, title = GetAddOnInfo("YurrCombatLogOptions")
		if (not title) then title = "YurrCombatLogOptions" end
		if reason then
			self:Print(format("%s (Enable) LoadOnDemand Error - %s", title, reason))
		end
    end
	if s then
		LibStub("AceConfigDialog-3.0"):Close("YurrCombatLog")
	else
		LibStub("AceConfigDialog-3.0"):Open("YurrCombatLog")
	end
	s = not s
end

local ldb = LibStub("LibDataBroker-1.1")
local dataobj = ldb:NewDataObject("YurrCombatLogOptions", {type = "launcher", icon = "Interface\\Icons\\Ability_MeleeDamage", text = "YurrCombatLog"})

function dataobj.OnTooltipShow(self)
	self:AddLine(dataobj.text)
	self:AddLine(format("|cffffff00%s", L.leftClick))
end
dataobj.OnClick = function() YurrCombatLog:showoptionsgui() end

local orgFCF_DockUpdate = FCF_DockUpdate
function YurrCombatLog:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT")
	self:RegisterEvent("ADDON_LOADED")
	
	if self.db.profile.contextMenu then self:InsertContext() end
end

-- ninjad from Prat_ArmoryLink
local popupList
function YurrCombatLog:UnitPopup_Click(info)
	local name = UIDROPDOWNMENU_INIT_MENU.name
	local button = info.value
	local click = {
		["ADD_TO_YCL"] = function()
			self.db.profile.watches[name] = UnitGUID(name)
			if IsAddOnLoaded("YurrCombatLogOptions") then
				local module = self:GetModule("YurrCombatLogOptions")
				local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
				options.args.watches.args[name] = module:createWatch(name, UnitGUID(name))
				AceConfigRegistry:NotifyChange("YurrCombatLog")
			end
			self:setFilter()
		end,
		["REMOVE_FROM_YCL"] = function()
			self.db.profile.watches[name] = nil
			if IsAddOnLoaded("YurrCombatLogOptions") then
				local module = self:GetModule("YurrCombatLogOptions")
				local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
				options.args.watches.args[name] = nil
				AceConfigRegistry:NotifyChange("YurrCombatLog")
			end
			self:setFilter()
		end
	}
	if click[button] then click[button]() end
end
function YurrCombatLog:UnitPopup_ShowMenu(dropdownMenu, which, unit, name, userData, ...)
	for i=1, UIDROPDOWNMENU_MAXBUTTONS do
		button = _G[format("DropDownList%dButton%d", UIDROPDOWNMENU_MENU_LEVEL, i)];

		-- Patch our handler function back in
		if  button.value == "ADD_TO_YCL" then
		    button.func = UnitPopupButtons["ADD_TO_YCL"].func
		elseif  button.value == "REMOVE_FROM_YCL" then
		    button.func = UnitPopupButtons["REMOVE_FROM_YCL"].func
		end
	end
end
function YurrCombatLog:InsertContext()
	-- ninjad from Prat_ArmoryLink
	self:SecureHook("UnitPopup_ShowMenu")
	UnitPopupButtons['ADD_TO_YCL'] = { 
		text = L["Watch in (Yurr)CombatLog"], 
		dist = 0, 
		func = function(info) YurrCombatLog:UnitPopup_Click(info) end,
	}
	UnitPopupButtons['REMOVE_FROM_YCL'] = { 
		text = L["Remove watch from (Yurr)CombatLog"],
		dist = 0, 
		func = function(info) YurrCombatLog:UnitPopup_Click(info) end,
	}
	popupList = { 
		'PARTY',        -- party frame.
		'FRIEND',       -- friend frame
		'SELF',         -- ?
		'PLAYER',       -- ?
		--'RAID',         -- raid frames
		'RAID_PLAYER',  -- player's raid frame?
		'TEAM'          -- arena team
	}
	for key,val in pairs(popupList) do
		tinsert(UnitPopupMenus[val], #UnitPopupMenus[val]-1,'ADD_TO_YCL')
		tinsert(UnitPopupMenus[val], #UnitPopupMenus[val]-1,'REMOVE_FROM_YCL')
	end
end
function YurrCombatLog:RemoveContext()

	for j = 1, #popupList do
		local t = popupList[j]
		for i = 1, #UnitPopupMenus[t] do
			if UnitPopupMenus[t][i] == "ADD_TO_YCL" then
				tremove(UnitPopupMenus[t], i)
				break
			end
		end
	end
	for j = 1, #popupList do
		local t = popupList[j]
		for i = 1, #UnitPopupMenus[t] do
			if UnitPopupMenus[t][i] == "REMOVE_FROM_YCL" then
				tremove(UnitPopupMenus[t], i)
				break
			end
		end
	end
	self:Unhook("UnitPopup_ShowMenu")
end

function YurrCombatLog:Blizzard_CombatLog_Refilter()
	local count = CombatLogGetNumEntries();
	
	COMBATLOG:SetMaxLines(COMBATLOG_MESSAGE_LIMIT);
	_G[self.db.profile.outputFrame]:SetMaxLines(COMBATLOG_MESSAGE_LIMIT)

	count = max(1, min(count, COMBATLOG_MESSAGE_LIMIT));

	CombatLogSetCurrentEntry(0);
	
	_G[self.db.profile.outputFrame]:Clear()
	
	CombatLogQuickButtonFrameProgressBar:SetMinMaxValues(0, count);	
	CombatLogQuickButtonFrameProgressBar:SetValue(0);
	CombatLogQuickButtonFrameProgressBar:Show();

	CombatLogUpdateFrame.refiltering = true;
	CombatLogUpdateFrame:SetScript("OnUpdate", Blizzard_CombatLog_RefilterUpdate)	
	
	Blizzard_CombatLog_RefilterUpdate()
end
function YurrCombatLog:Blizzard_CombatLog_RefilterUpdate()
	local valid = CombatLogGetCurrentEntry();
	
	local total = 0;
	while (valid and total < COMBATLOG_LIMIT_PER_FRAME) do 
		local YCLtext = self:processEvent("COMBAT_LOG_EVENT", CombatLogGetCurrentEntry())
		_G[self.db.profile.outputFrame]:AddMessage( YCLtext, 1, 1, 1, nil, true );
		
		-- count can be 
		--  positive to advance from oldest to newest
		--  negative to advance from newest to oldest
		valid = CombatLogAdvanceEntry(-1)
		total = total + 1;
	end
	
	CombatLogQuickButtonFrameProgressBar:SetValue(CombatLogQuickButtonFrameProgressBar:GetValue() + total);
	
	if ( not valid or (CombatLogQuickButtonFrameProgressBar:GetValue() >= COMBATLOG_MESSAGE_LIMIT) ) then
		CombatLogUpdateFrame.refiltering = false
		CombatLogUpdateFrame:SetScript("OnUpdate", nil)
		CombatLogQuickButtonFrameProgressBar:Hide();
	end
end
function YurrCombatLog:CombatLog_AddEvent()
end
function YurrCombatLog:FCF_DockUpdate()
	orgFCF_DockUpdate()
	self:Blizzard_CombatLog_AdjustCombatLogHeight()
end
function YurrCombatLog:Blizzard_CombatLog_AdjustCombatLogHeight()
	local uiScale = tonumber(GetCVar("uiScale"));
	local quickButtonHeight = 0
	local yOffset = (2 + quickButtonHeight*uiScale) / uiScale;
	local xOffset = 1 + uiScale;
	local combatLogBackground = _G[format("%sBackground", COMBATLOG:GetName())];
	combatLogBackground:SetPoint("TOPLEFT", COMBATLOG, "TOPLEFT", -xOffset, yOffset);
	combatLogBackground:SetPoint("TOPRIGHT", COMBATLOG, "TOPRIGHT", xOffset, yOffset);
end
function YurrCombatLog:CombatLogQuickButtonFrame_Custom_OnShow(frame)
	frame:Hide()
end

function YurrCombatLog:setFilter()
	CombatLogResetFilter()
	
	if next(self.db.profile.watches) == nil then
		for k,v in ipairs(self.db.profile.filters) do
			if v then
				local events = ""
				local sUnit = 0
				local dUnit = 0
				if v.active then
					for k2,v2 in pairs(v) do
						if k2 ~= "name" and k2 ~= "active" and k2 ~= "sourceUnit" and k2 ~= "destUnit" and v2 then
							events = format("%s, %s", events, k2)
						end
					end
					for k2,v2 in pairs(v.sourceUnit) do
						if v2 then
							sUnit = bit.bor(sUnit, _G[k2])
						end
					end
					for k2,v2 in pairs(v.destUnit) do
						if v2 then
							dUnit = bit.bor(dUnit, _G[k2])
						end
					end
					if events == "" then sUnit = nil end
					if sUnit == 0 then sUnit = nil end
					if dUnit == 0 then dUnit = nil end
					CombatLogAddFilter(events, sUnit, dUnit)
				end
			end
		end
	else
		for k,v in pairs(self.db.profile.watches) do
			CombatLogAddFilter("ENVIRONMENTAL_DAMAGE, SWING_DAMAGE, RANGE_DAMAGE, SPELL_DAMAGE, SPELL_HEAL, SPELL_PERIODIC_HEAL, SPELL_PERIODIC_DAMAGE, DAMAGE_SHIELD, SPELL_INSTAKILL SPELL_AURA_APPLIED, SPELL_AURA_REMOVED, UNIT_DIED", nil, v)
		end
	end
	self:Blizzard_CombatLog_Refilter()
end

function YurrCombatLog:AffiliationColor(flags)
	local color
	local colors = self.db.profile.unitColors
	for mask in pairs(colors) do
		if (CombatLog_Object_IsA (flags, mask))then
			color = colors[mask]
			break
		end
	end
	return color
end

function YurrCombatLog:ColorSpellBySchool(school, aura, suffix)
	local colors = self.db.profile.spellColors
	return spellList[suffix] or spellList[aura] or colors[school]
end

function YurrCombatLog:DamageResult(resisted, blocked, absorbed, critical, glancing, crushing, healAbsorb)
	local resultStr
	local formatting = self.db.profile.format
	
	if ( resisted or blocked or absorbed or critical or glancing or crushing or healAbsorb ) then
		
		if ( resisted ) then
			if ( resisted < 0 ) then	--Its really a vulnerability
				resultStr = format(formatting.VULNERABILITY, -resisted)
			else
				resultStr = format(formatting.RESIST, resisted)
			end
		end
		if ( blocked ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..format(formatting.BLOCK_VALUE, blocked)
			else
				resultStr = format(formatting.BLOCK_VALUE, blocked)
			end
		end
		if ( absorbed and absorbed > 0 or healAbsorb and healAbsorb > 0 ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..format(formatting.ABSORB, absorbed)
			else
				resultStr = format(formatting.ABSORB, absorbed or healAbsorb)
			end
		end
		if ( glancing ) then
			if ( resultStr ) then
				resultStr = format("%s %s", resultStr, formatting.GLANCING)
			else
				resultStr = formatting.GLANCING;
			end
		end
		if ( crushing ) then
			if ( resultStr ) then
				resultStr = format("%s %s", resultStr, formatting.CRUSHING)
			else
				resultStr = formatting.CRUSHING
			end
		end
		if ( critical ) then
			local critString = formatting.CRITICAL
			if ( spellId ) then
				critString = formatting.CRITICAL_SPELL
			end
			if ( resultStr ) then
				resultStr = format("%s %s", resultStr, critString)
			else
				resultStr = critString
			end
		end
	end
	
	return resultStr

end

function YurrCombatLog:COMBAT_LOG_EVENT(...)
	local t = self:processEvent(...)
	_G[self.db.profile.outputFrame]:AddMessage(t)
end

function YurrCombatLog:ADDON_LOADED(self, name)
	if name == "Blizzard_CombatLog" then
		YurrCombatLog:RawHook("Blizzard_CombatLog_Refilter", true)
		YurrCombatLog:RawHook("Blizzard_CombatLog_RefilterUpdate", true)
		YurrCombatLog:RawHook("CombatLog_AddEvent", true)
		YurrCombatLog:RawHook("Blizzard_CombatLog_ApplyFilters", "setFilter", true)
		YurrCombatLog:RawHook("FCF_DockUpdate", true)
		YurrCombatLog:HookScript(CombatLogQuickButtonFrame_Custom, "OnShow", "CombatLogQuickButtonFrame_Custom_OnShow")
		CombatLogQuickButtonFrame_Custom:Hide()
		
		CombatLogQuickButtonFrameProgressBar = _G.CombatLogQuickButtonFrame_CustomProgressBar
	end
end

function YurrCombatLog:processEvent(...)
	--Debug(...)
	
	local profile = self.db.profile
	local braceColors = profile.braceColors
	local braces = profile.braces
	
	local _, timestamp, event, srcGUID, srcUnitName, srcFlags, destGUID, destUnitName, destFlags = ...
	
	local timestampStr
	
	local actionString
	
	local srcUnitNameString
	
	local prefixString
	
	local spellName
	local spellNameString
	
	local suffixString
	
	local destUnitNameString
	
	local amount
	local auraStack
	local overkill
	local overhealing
	local healAbsorb
	local school
	local resisted
	local blocked
	local absorbed
	local critical
	local glancing
	local crushing
	local missType
	local amountMissed
	local powerType
	local extraSpellName
	local extraSchool
	local auraType
	local failedType
	local itemID
	local itemName
	local amountExtraAttack
		
	local amountString
	local powerString
	local schoolStr
	local resultString
	
	local finalString
	
	if ( srcUnitName and CombatLog_Object_IsA(srcFlags, COMBATLOG_FILTER_MINE) ) then
		srcUnitName = UNIT_YOU_SOURCE;
	end
	if ( destUnitName and CombatLog_Object_IsA(destFlags, COMBATLOG_FILTER_MINE) ) then
		destUnitName = UNIT_YOU_DEST;
	end
	
	local base = 9
	local offset
	
	-- Prepare strings
	local prefix = strsub(event, 1, 5)
	local suffix = strsub(event, 6)
	
	-- Special events
	if event == "UNIT_DIED" then
		actionString = TUTORIAL_TITLE25 .. ": "
	elseif event == "UNIT_DESTROYED" then
		actionString = ACTION_UNIT_DESTROYED .. ": "
	elseif event == "UNIT_DISSIPATES" then
		actionString = L["Dissipated"] .. ": "
	elseif event == "ENCHANT_APPLIED" or event == "ENCHANT_REMOVED" then
		actionString = ENSCRIBE .. ": "
		offset = 0
		spellName, itemID, itemName = select(base + offset + 1, ...)
		school = 1
	elseif event == "PARTY_KILL" then
		spellName = ACTION_PARTY_KILL
		school = 1
	elseif event == "DAMAGE_SHIELD" or event == "DAMAGE_SPLIT" then
		offset = 3
		prefix = "SPELL"
		suffix = "_DAMAGE"
	elseif event == "DAMAGE_SHIELD_MISSED" then
		offset = 3
		prefix = "SPELL"
		suffix = "_MISSED"
	end
	
	if prefix == "SWING" then
		offset = 0
		spellName = MELEE
	elseif prefix == "SPELL" or prefix == "RANGE" then
		offset = 3
		spellName, school = select(11, ...)
		-- (Temporary) fix for death knight gladiator gloves generating an energize event with a zero(not nil) spellschool
		if school == 0 then school = 1 end
	elseif prefix == "ENVIR" then
		offset = 1
		srcGUID = destGUID
		srcUnitName = destUnitName
		srcFlags = destFlags
		destGUID = nil
		destUnitName = nil
		destFlags = nil
		local environmentalType = select(base + offset, ...)
		spellName = L[environmentalType]
		suffix = "_DAMAGE"
	end
	if suffix == "_DAMAGE" or suffix == "_PERIODIC_DAMAGE" or suffix == "_BUILDING_DAMAGE" then
		amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing = select(base + offset + 1, ...)
	elseif suffix == "_MISSED" or suffix == "_PERIODIC_MISSED" or suffix == "_BUILDING_MISSED" then
		missType, amountMissed = select(base + offset + 1, ...)
		school = school or 1
	elseif suffix == "_HEAL" or suffix == "_PERIODIC_HEAL" or suffix == "_BUILDING_HEAL" then
		amount, overhealing, healAbsorb, critical = select(base + offset + 1, ...)
	elseif suffix == "_ENERGIZE" or suffix == "_PERIODIC_ENERGIZE" then
		amount, powerType = select(base + offset + 1, ...)
	elseif suffix == "_DRAIN" or suffix == "_LEECH" or suffix == "_PERIODIC_DRAIN" or suffix == "_PERIODIC_LEECH" then
		amount, powerType, extraAmount = select(base + offset + 1, ...)
	elseif suffix == "_INTERRUPT" or suffix == "_DISPEL" or suffix == "_DISPEL_FAILED" or suffix == "_STOLEN" or suffix == "_AURA_BROKEN_SPELL" then
		--extraSpellID = select(base + offset + 1, ...)
		extraSpellName, extraSchool, auraType = select(base + offset + 2, ...)
	elseif suffix == "_EXTRA_ATTACKS" then
		amountExtraAttack = select(base + offset + 1, ...)
		spellName =  format(L.extraAttack, amountExtraAttack, spellName)
		school = 1
	elseif suffix == "_AURA_APPLIED" or suffix == "_AURA_REMOVED" or suffix == "_AURA_APPLIED_DOSE" or suffix == "_AURA_REMOVED_DOSE" or suffix == "_AURA_REFRESH" or suffix == "_AURA_BROKEN" then
		auraType, auraStack = select(base + offset + 1, ...)
	elseif suffix == "_CAST_FAILED" then
		failedType = select(base + offset + 1, ...)
	end
	
	if profile.blacklist[spellName] then return end
	if profile.enableWhitelist and not profile.whitelist[spellName] then return end
	
	-- Shortening useless information
	if srcUnitName == spellName then
		if not (CombatLog_Object_IsA(srcFlags, COMBATLOG_OBJECT_TYPE_PLAYER)) then
			if spellName == destUnitName then
				destUnitName = nil
			else
				spellName = nil
			end
		end
	end
	
	-- Process strings
	if profile.timestamp then
		local stamp
		local pre
		local post
		if profile.colorTimestamp then
			stamp = "|cFF" .. profile.otherColors.timestampColor .. date(profile.timestampFormat, timestamp) .. "|r" -- SOME GARBAGE HERE
		else
			stamp = date(profile.timestampFormat, timestamp) -- SOME GARBAGE HERE
		end
		if profile.braceTimestamp then
			if profile.colorTimestampBrace then
				pre = "|cFF" .. braceColors.Timestamp .. braces.preTimestamp .. "|r"
				post = "|cFF" .. braceColors.Timestamp .. braces.postTimestamp .. "|r"
			else
				pre = braces.preTimestamp
				post = braces.postTimestamp
			end
			timestampStr = pre .. stamp .. post
		else
			timestampStr = stamp
		end
	else
		timestampStr = ""
	end
	
	if srcUnitName then
		local UnitStr
		local pre
		local post
		if profile.colorSrc then
			local color
			local classColor
			if profile.colorUnitByClass then
				local _, class = GetPlayerInfoByGUID(srcGUID)
				if class then
					classColor = classColors[class]
				end
			end
			color = self:AffiliationColor(srcFlags)
			UnitStr = "|cFF" .. (classColor or color) .. srcUnitName .. "|r"
		else
			UnitStr = srcUnitName
		end
		if profile.braceSrcUnit then
			if profile.colorSrcBraces then
				pre = "|cFF" .. braceColors.Source .. braces.preSource .. "|r"
				post = "|cFF" .. braceColors.Source .. braces.postSource .. "|r"
			else
				pre = braces.preSource
				post = braces.postSource
			end
		else
			pre = ""
			post = ""
		end
		srcUnitNameString = pre .. UnitStr .. post .. " "
	else
		srcUnitNameString = ""
	end
	
	if profile.prefix then
		local str = prefixList[suffix]
		if profile.colorPrefix and str then
			local color = self:ColorSpellBySchool(school, auraType, suffix)
			prefixString = "|cFF" .. color .. str .. "|r "
		elseif str then
			prefixString = str .. " "
		else
			prefixString = ""
		end
	else
		prefixString = ""
	end
	
	if spellName then
		local coloredSpell
		local pre
		local post
		local stackedSpell
		if auraStack then
			stackedSpell = spellName .. "(" .. auraStack .. ")"
		else
			stackedSpell = spellName
		end
		
		if profile.colorSpell then
			local color = self:ColorSpellBySchool(school, auraType, suffix)
			coloredSpell = "|cFF" .. color .. stackedSpell .. "|r"
		else
			coloredSpell = stackedSpell
		end
		if profile.braceSpell then
			if profile.colorSpellBraces then
				pre = "|cFF" .. braceColors.Spell .. braces.preSpell .. "|r"
				post = "|cFF" .. braceColors.Spell .. braces.postSpell .. "|r"
			else
				pre = braces.preSpell
				post = braces.postSpell
			end
		else
			pre = ""
			post = ""
		end
		spellNameString = pre .. coloredSpell .. post .. " "
	else
		spellNameString = ""
	end
	
	if profile.suffixString then
		local str = suffixList[suffix] or suffixList[missType] or failedType
		local result
		
		if str then
			if profile.colorSuffix then
				local color = self:ColorSpellBySchool(school, aura, suffix)
				suffixString = "|cFF" .. color .. str .. "|r "
			else
				suffixString = str .. " "
			end
		else
			suffixString = ""
		end
	else
		suffixString = ""
	end
	
	if destUnitName then
		local UnitStr
		local pre
		local post
		if profile.colorDest then
			local color
			local _, class = GetPlayerInfoByGUID(destGUID)
			local classColor
			if profile.colorUnitByClass and class then
				classColor = classColors[class]
			else
				color = self:AffiliationColor(destFlags)
			end
			UnitStr = "|cFF" .. (classColor or color) .. destUnitName .. "|r"
		else
			UnitStr = destUnitName
		end
		if profile.braceDestUnit then
			if profile.colorDestBraces then
				pre = "|cFF" .. braceColors.Dest .. braces.preDest .. "|r"
				post = "|cFF" .. braceColors.Dest .. braces.postDest .. "|r"
			else
				pre = braces.preDest
				post = braces.postDest
			end
			destUnitNameString = pre .. UnitStr .. post .. " "
		else
			destUnitNameString = UnitStr .. " "
		end
	else
		destUnitNameString = ""
	end
	
	if amount then
		local amountResult
		local AmountStr
		local preBrace
		local postBrace
		local preCrit
		local postCrit
		local preDot
		local postDot
		
		if profile.overhealing and type(overhealing) == "number" and overhealing > 0 then
			--amountResult = format("%d(+%d)", amount - overhealing, overhealing) -- SOME GARBAGE HERE
			amountResult = amount - overhealing .. "(+" .. overhealing .. ")"
		elseif profile.overkill and type(overkill) == "number" and overkill > 0 then
			--amountResult = format("%d(+%d)", amount - overkill, overkill) -- SOME GARBAGE HERE
			amountResult = amount - overkill .. "(+" .. overkill .. ")"
		else
			amountResult = amount
		end
		
		if suffix == "_ENERGIZE" or suffix == "_PERIODIC_ENERGIZE" then
			amountResult = "+ " .. amountResult
		end
		
		if profile.colorAmount and school then
			local color = self:ColorSpellBySchool(school, auraType, suffix)
			AmountStr = "|cFF" .. color .. amountResult .. "|r"
		else
			AmountStr = amountResult
		end
		
		if profile.braceAmount then
			if profile.colorAmountBraces then
				preBrace = "|cFF" .. braceColors.Amount .. braces.preAmount .. "|r"
				postBrace = "|cFF" .. braceColors.Amount .. braces.postAmount .. "|r"
			else
				preBrace = braces.preAmount
				postBrace = braces.postAmount
			end
		else
			preBrace = ""
			postBrace = ""
		end
		
		if profile.markCrits and critical then
			preCrit = braces.preCrit
			postCrit = braces.postCrit
		else
			preCrit = ""
			postCrit = ""
		end
		
		if profile.braceAmountDots and (suffix == "_PERIODIC_DAMAGE" or suffix == "_PERIODIC_HEAL") then
			if profile.colorAmountDotBraces then
				preDot = "|cFF" .. braceColors.AmountDots .. braces.preAmountDots .. "|r"
				postDot = "|cFF" .. braceColors.AmountDots .. braces.postAmountDots .. "|r"
			else
				preDot = braces.preAmountDots
				postDot = braces.postAmountDots
			end
		else
			preDot = ""
			postDot = ""
		end
		
		amountNameString = preBrace .. preCrit .. preDot .. AmountStr .. postDot .. postCrit .. postBrace .. " "
	else
		amountNameString = ""
	end
	
	if extraSpellName then
		local color = self:ColorSpellBySchool(extraSchool, auraType)
		amountNameString = "|cFF" .. color .. extraSpellName .. "|r "
	end
	if itemName then
		amountNameString = itemName
	end
	
	if profile.powerType and powerType then
		powerString = powerTypes[powerType] .. " "
	else
		powerString = ""
	end
	
	if profile.schoolText and school then
		local coloredSchool
		local preShool
		local postSchool
		if profile.colorSchoolText then
			local color = self:ColorSpellBySchool(school)
			coloredSchool = "|cFF" .. color .. schools[school] .. "|r"
		else
			coloredSchool = schools[school]
		end
		if profile.braceSchoolText then
			if profile.colorSchoolTextBraces then
				local color = self:ColorSpellBySchool(school)
				preSchool = "|cFF" .. color .. braces.preSchoolText .. "|r"
				postSchool = "|cFF" .. color .. braces.postSchoolText .. "|r"
			else
				preSchool = braces.preSchoolText
				postSchool = braces.postSchoolText
			end
		else
			preSchool = ""
			postSchool = ""
		end
		schoolStr = preSchool .. coloredSchool .. postSchool .. " "
	else
		schoolStr = ""
	end
	
	if profile.resultString then
		if missType == "RESIST" then
			resisted = amountMissed
		elseif missType == "BLOCK" then
			blocked = amountMissed
		elseif missType == "ABSORB" then
			absorbed = amountMissed
		elseif missType ~= "MISS" then
			resultString = profile.format[missType]
		end
			
		if suffix == "_DISPEL_FAILED" then
			resultString = L.failed
		end
		
		if not resultString then resultString = self:DamageResult(resisted, blocked, absorbed, critical, glancing, crushing, healAbsorb) or "" end
	else
		resultString = ""
	end

	finalString = timestampStr .. (actionString or "") .. srcUnitNameString .. prefixString .. spellNameString .. suffixString .. destUnitNameString .. amountNameString .. powerString .. schoolStr .. resultString
	-- if FilterText then
		-- if string.find(finalString, FilterText) then
			-- return finalString
		-- end
	-- else
		return finalString
	-- end
		
end

